package sun.awt;

import java.awt.IllegalComponentStateException;
import java.awt.Container;
import java.awt.Window;
import java.awt.Point;
import java.awt.Cursor;
import java.awt.event.InvocationEvent;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.event.InputEvent;

public abstract class GlobalCursorManager
{
    private final GlobalCursorManager.NativeUpdater nativeUpdater;
    private long lastUpdateMillis;
    private final Object lastUpdateLock;
    
    public void updateCursorImmediately() {
        synchronized (this.nativeUpdater) {
            this.nativeUpdater.pending = false;
        }
        this._updateCursor(false);
    }
    
    public void updateCursorImmediately(InputEvent inputEvent) {
        int n;
        synchronized (this.lastUpdateLock) {
            n = (inputEvent.getWhen() >= this.lastUpdateMillis);
        }
        if (!(n == 0)) {
            this._updateCursor(true);
        }
    }
    
    public void updateCursorLater(Component component) {
        this.nativeUpdater.postIfNotPending(component, new InvocationEvent(Toolkit.getDefaultToolkit(), this.nativeUpdater));
    }
    
    protected GlobalCursorManager() {
        this.nativeUpdater = new GlobalCursorManager.NativeUpdater();
        this.lastUpdateLock = new Object();
    }
    
    protected abstract void setCursor(Component p0, Cursor p1, boolean p2);
    
    protected abstract void getCursorPos(Point p0);
    
    protected abstract Point getLocationOnScreen(Component p0);
    
    protected abstract Component findHeavyweightUnderCursor(boolean p0);
    
    private void _updateCursor(boolean b) {
        synchronized (this.lastUpdateLock) {
            this.lastUpdateMillis = System.currentTimeMillis();
        }
        Point point = null;
        try {
            Component heavyweightUnderCursor = this.findHeavyweightUnderCursor(b);
            if (heavyweightUnderCursor == null) {
                this.updateCursorOutOfJava();
                return;
            }
            if ((heavyweightUnderCursor instanceof Window)) {
                point = AWTAccessor.getComponentAccessor().getLocation(heavyweightUnderCursor);
            }
            else if ((heavyweightUnderCursor instanceof Container)) {
                point = this.getLocationOnScreen(heavyweightUnderCursor);
            }
            if (point != null) {
                Point o = new Point();
                this.getCursorPos(o);
                Component component = AWTAccessor.getContainerAccessor().findComponentAt((Container)heavyweightUnderCursor, o.x - point.x, o.y - point.y, false);
                if (component != null) {
                    heavyweightUnderCursor = component;
                }
            }
            this.setCursor(heavyweightUnderCursor, AWTAccessor.getComponentAccessor().getCursor(heavyweightUnderCursor), b);
        }
        catch (IllegalComponentStateException ex) {}
    }
    
    protected void updateCursorOutOfJava() {
    }
    
    static void access$000(GlobalCursorManager globalCursorManager, boolean b) {
        globalCursorManager._updateCursor(b);
    }
    
    class NativeUpdater implements Runnable
    {
        boolean pending;
        
        NativeUpdater() {
            this.pending = false;
        }
        
        @Override
        public void run() {
            boolean n = false;
            synchronized (this) {
                if (this.pending) {
                    this.pending = false;
                    n = true;
                }
            }
            if (!(n == 0)) {
                GlobalCursorManager.access$000(GlobalCursorManager.this, false);
            }
        }
        
        public void postIfNotPending(Component component, InvocationEvent invocationEvent) {
            int n = 0;
            synchronized (this) {
                if (!this.pending) {
                    n = (this.pending = true);
                }
            }
            if (n != 0) {
                SunToolkit.postEvent(SunToolkit.targetToAppContext(component), invocationEvent);
            }
        }
    }
}
